@@ -1,7 +1,6 @@
  /*
     libparted - a library for manipulating disk partitions
-    Copyright (C) 1999, 2000, 2001, 2002, 2003, 2005, 2007, 2008
-                  Free Software Foundation, Inc.
+    Copyright (C) 1999-2003, 2005, 2007-2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -148,8 +147,15 @@ ped_disk_probe (PedDevice* dev)
         ped_exception_fetch_all ();
         for (walk = ped_disk_type_get_next (NULL); walk;
              walk = ped_disk_type_get_next (walk))
+          {
+                if (getenv ("PARTED_DEBUG")) {
+                        fprintf (stderr, "probe label: %s\n",
+                                 walk->name);
+                        fflush (stderr);
+                }
                 if (walk->ops->probe (dev))
                         break;
+          }
 
         if (ped_exception)
                 ped_exception_catch ();
@@ -345,18 +351,19 @@ ped_disk_new_fresh (PedDevice* dev, const PedDiskType* type)
 	PED_ASSERT (dev != NULL, return NULL);
 	PED_ASSERT (type != NULL, return NULL);
 	PED_ASSERT (type->ops->alloc != NULL, return NULL);
+	PedCHSGeometry*	bios_geom = &dev->bios_geom;
+	PED_ASSERT (bios_geom->sectors != 0, return NULL);
+	PED_ASSERT (bios_geom->heads != 0, return NULL);
 
 	disk = type->ops->alloc (dev);
 	if (!disk)
        		goto error;
 	_disk_pop_update_mode (disk);
-	PED_ASSERT (disk->update_mode == 0, goto error_destroy_disk);
+	PED_ASSERT (disk->update_mode == 0, ignored);
 
 	disk->needs_clobber = 1;
 	return disk;
 
-error_destroy_disk:
-	ped_disk_destroy (disk);
 error:
 	return NULL;
 }
@@ -489,9 +496,25 @@ error:
 int
 ped_disk_commit (PedDisk* disk)
 {
+        /* Open the device here, so that the underlying fd is not closed
+           between commit_to_dev and commit_to_os (closing causes unwanted
+           udev events to be sent under Linux). */
+	if (!ped_device_open (disk->dev))
+		goto error;
+
 	if (!ped_disk_commit_to_dev (disk))
-		return 0;
-	return ped_disk_commit_to_os (disk);
+		goto error_close_dev;
+
+	if (!ped_disk_commit_to_os (disk))
+		goto error_close_dev;
+
+	ped_device_close (disk->dev);
+	return 1;
+
+error_close_dev:
+	ped_device_close (disk->dev);
+error:
+	return 0;
 }
 
 /**
@@ -1693,31 +1716,6 @@ _check_extended_partition (PedDisk* disk, PedPartition* part)
 	return 1;
 }
 
-static PedSector
-_partition_max_start (char const *label_type)
-{
-  /* List partition table names (a la disk->type->name) for which
-     the partition length, in sectors, must fit in 32 bytes.  */
-  static char const *const max_32[] = {"msdos", "dvh"};
-  unsigned int i;
-
-  for (i = 0; i < sizeof max_32 / sizeof *max_32; i++)
-    if (strcmp (label_type, max_32[i]) == 0)
-      return UINT32_MAX;
-
-  return TYPE_MAXIMUM (PedSector);
-}
-
-static PedSector
-_partition_max_len (char const *label_type)
-{
-  /* NOTE: for now, they happen to be the same, so don't
-     duplicate needlessly.  Of course, if there's some format
-     with different length and starting sector limits, then
-     these functions will diverge.  */
-  return _partition_max_start (label_type);
-}
-
 static int
 _check_partition (PedDisk* disk, PedPartition* part)
 {
@@ -1758,44 +1756,9 @@ _check_partition (PedDisk* disk, PedPartition* part)
 		return 0;
 	}
 
-	if (!(part->type & PED_PARTITION_METADATA)) {
-		char const *label_type = disk->type->name;
-		/* Enforce some restrictions inherent in the DOS
-		   partition table format.  Without these, one would be able
-		   to create a 2TB partition (or larger), and it would work,
-		   but only until the next reboot.  This was insidious: the
-		   too-large partition would work initially, because with
-		   Linux-2.4.x and newer we set the partition start sector
-		   and length (in sectors) accurately and directly via the
-		   BLKPG ioctl.  However, only the last 32 bits of each
-		   number would be written to the partition table, and the
-		   next time the system would read/use those corrupted numbers
-		   it would usually complain about an invalid partition.
-                   The same applies to the starting sector number.  */
-
-		if (part->geom.length > _partition_max_len (label_type)) {
-			ped_exception_throw (
-				PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				_("partition length of %jd sectors exceeds the "
-                                  "%s-partition-table-imposed maximum of %jd"),
-				part->geom.length,
-                                label_type,
-                                _partition_max_len (label_type));
+	if (!(part->type & PED_PARTITION_METADATA))
+		if (!disk->type->ops->partition_check(part))
 			return 0;
-		}
-
-		/* The starting sector number must fit in 32 bytes.  */
-		if (part->geom.start > _partition_max_start (label_type)) {
-			ped_exception_throw (
-				PED_EXCEPTION_ERROR, PED_EXCEPTION_CANCEL,
-				_("starting sector number, %jd exceeds the"
-                                  " %s-partition-table-imposed maximum of %jd"),
-                                part->geom.start,
-                                label_type,
-                                _partition_max_start (label_type));
-			return 0;
-		}
-	}
 
 	return 1;
 }
@@ -1884,15 +1847,11 @@ ped_disk_remove_partition (PedDisk* disk, PedPartition* part)
 	PED_ASSERT (part != NULL, return 0);
 
 	_disk_push_update_mode (disk);
-	PED_ASSERT (part->part_list == NULL, goto error);
+	PED_ASSERT (part->part_list == NULL, ignored);
 	_disk_raw_remove (disk, part);
 	_disk_pop_update_mode (disk);
 	ped_disk_enumerate_partitions (disk);
 	return 1;
-
-error:
-	_disk_pop_update_mode (disk);
-	return 0;
 }
 
 static int
