@@ -40,6 +40,8 @@
 #include <stdbool.h>
 #include "xalloc.h"
 
+#include "pt-tools.h"
+
 #if ENABLE_NLS
 #  include <libintl.h>
 #  define _(String) gettext (String)
@@ -430,7 +432,6 @@ gpt_probe (const PedDevice * dev)
 {
 	GuidPartitionTableHeader_t* gpt = NULL;
         uint8_t* pth_raw = ped_malloc (pth_get_size (dev));
-	LegacyMBR_t legacy_mbr;
 	int gpt_sig_found = 0;
 
 	PED_ASSERT (dev != NULL, return 0);
@@ -451,26 +452,31 @@ gpt_probe (const PedDevice * dev)
 	if (!gpt_sig_found)
 		return 0;
 
-	if (ped_device_read(dev, &legacy_mbr, 0, GPT_HEADER_SECTORS)) {
-		if (!_pmbr_is_valid (&legacy_mbr)) {
-			int ex_status = ped_exception_throw (
-				PED_EXCEPTION_WARNING,
-				PED_EXCEPTION_YES_NO,
-			_("%s contains GPT signatures, indicating that it has "
-			  "a GPT table.  However, it does not have a valid "
-			  "fake msdos partition table, as it should.  Perhaps "
-			  "it was corrupted -- possibly by a program that "
-			  "doesn't understand GPT partition tables.  Or "
-			  "perhaps you deleted the GPT table, and are now "
-			  "using an msdos partition table.  Is this a GPT "
-			  "partition table?"),
-				dev->path);
-			if (ex_status == PED_EXCEPTION_NO)
-				return 0;
-		}
+
+	void *label;
+	if (!ptt_read_sector (dev, 0, &label))
+		return 0;
+
+        int ok = 1;
+	if (!_pmbr_is_valid ( (const LegacyMBR_t *) label)) {
+		int ex_status = ped_exception_throw (
+			PED_EXCEPTION_WARNING,
+			PED_EXCEPTION_YES_NO,
+		_("%s contains GPT signatures, indicating that it has "
+		  "a GPT table.  However, it does not have a valid "
+		  "fake msdos partition table, as it should.  Perhaps "
+		  "it was corrupted -- possibly by a program that "
+		  "doesn't understand GPT partition tables.  Or "
+		  "perhaps you deleted the GPT table, and are now "
+		  "using an msdos partition table.  Is this a GPT "
+		  "partition table?"),
+			dev->path);
+		if (ex_status == PED_EXCEPTION_NO)
+			ok = 0;
 	}
 
-	return 1;
+        free (label);
+	return ok;
 }
 
 #ifndef DISCOVER_ONLY
@@ -478,34 +484,31 @@ gpt_probe (const PedDevice * dev)
 static int
 gpt_clobber(PedDevice * dev)
 {
-	LegacyMBR_t pmbr;
-        uint8_t* zeroed_pth_raw = ped_malloc (pth_get_size (dev));
         uint8_t* pth_raw = ped_malloc (pth_get_size (dev));
 	GuidPartitionTableHeader_t* gpt;
 
 	PED_ASSERT (dev != NULL, return 0);
 
-	memset(&pmbr, 0, sizeof(pmbr));
-	memset(zeroed_pth_raw, 0, pth_get_size (dev));
-
         /*
          * TO DISCUSS: check whether checksum is correct?
          * If not, we might get a wrong AlternateLBA field and destroy
          * one sector of random data.
          */
 	if (!ped_device_read(dev, pth_raw,
-			     GPT_PRIMARY_HEADER_LBA, GPT_HEADER_SECTORS))
-                goto error_free;
+			     GPT_PRIMARY_HEADER_LBA, GPT_HEADER_SECTORS)) {
+                free (pth_raw);
+                return 0;
+        }
 
         gpt = pth_new_from_raw (dev, pth_raw);
+        free (pth_raw);
 
-	if (!ped_device_write(dev, &pmbr, GPT_PMBR_LBA, GPT_PMBR_SECTORS))
+	if (!ptt_clear_sectors(dev, GPT_PMBR_LBA, GPT_PMBR_SECTORS))
                 goto error_free_with_gpt;
-	if (!ped_device_write(dev, &zeroed_pth_raw,
-			      GPT_PRIMARY_HEADER_LBA, GPT_HEADER_SECTORS))
+	if (!ptt_clear_sectors(dev, GPT_PRIMARY_HEADER_LBA, GPT_HEADER_SECTORS))
                 goto error_free_with_gpt;
-	if (!ped_device_write(dev, &zeroed_pth_raw, dev->length - GPT_HEADER_SECTORS,
-			      GPT_HEADER_SECTORS))
+	if (!ptt_clear_sectors(dev, dev->length - GPT_HEADER_SECTORS,
+                               GPT_HEADER_SECTORS))
                 goto error_free_with_gpt;
 
 	if ((PedSector) PED_LE64_TO_CPU (gpt->AlternateLBA) < dev->length - 1) {
@@ -521,9 +524,6 @@ gpt_clobber(PedDevice * dev)
 
 error_free_with_gpt:
         pth_free (gpt);
-error_free:
-        free (pth_raw);
-        free (zeroed_pth_raw);
 	return 0;
 }
 #endif /* !DISCOVER_ONLY */
@@ -608,8 +608,10 @@ _header_is_valid (const PedDevice* dev, GuidPartitionTableHeader_t* gpt)
 	 * the SizeOfPartitionEntry must be a multiple of 8 and
 	 * no smaller than the size of the PartitionEntry structure.
 	 */
-	uint32_t sope = gpt->SizeOfPartitionEntry;
-	if (sope % 8 != 0 || sope < sizeof(GuidPartitionEntry_t) )
+	uint32_t sope = PED_LE32_TO_CPU (gpt->SizeOfPartitionEntry);
+	if (sope % 8 != 0
+            || sope < sizeof (GuidPartitionEntry_t)
+            || (UINT32_MAX >> 4) < sope)
 		return 0;
 
 	origcrc = gpt->HeaderCRC32;
@@ -911,7 +913,8 @@ gpt_read (PedDisk * disk)
 	if (!_parse_header (disk, gpt, &write_back))
 		goto error_free_gpt;
 
-	ptes_sectors = ped_div_round_up (gpt->SizeOfPartitionEntry
+	uint32_t p_ent_size = PED_LE32_TO_CPU (gpt->SizeOfPartitionEntry);
+	ptes_sectors = ped_div_round_up (p_ent_size
 					 * gpt_disk_data->entry_count,
 					 disk->dev->sector_size);
 
@@ -926,8 +929,7 @@ gpt_read (PedDisk * disk)
 
 	for (i = 0; i < gpt_disk_data->entry_count; i++) {
 		GuidPartitionEntry_t* pte
-		  = (GuidPartitionEntry_t*) ((char *)ptes + i
-					     * gpt->SizeOfPartitionEntry);
+		  = (GuidPartitionEntry_t*) ((char *)ptes + i * p_ent_size);
 		PedPartition* part;
 		PedConstraint* constraint_exact;
 
@@ -973,32 +975,34 @@ error:
 static int
 _write_pmbr (PedDevice * dev)
 {
-	LegacyMBR_t pmbr;
-
 	/* The UEFI spec is not clear about what to do with the following
 	   elements of the Protective MBR (pmbr): BootCode (0-440B),
 	   UniqueMBRSignature (440B-444B) and Unknown (444B-446B).
 	   With this in mind, we try not to modify these elements.  */
-	if (ped_device_read (dev, &pmbr, 0, GPT_PMBR_SECTORS) < GPT_PMBR_SECTORS)
-		memset (&pmbr, 0, sizeof(pmbr));
+	void *s0;
+	if (!ptt_read_sector (dev, 0, &s0))
+		return 0;
+	LegacyMBR_t *pmbr = s0;
 
-	/* Zero out all the legacy partitions.
-	   There are 4 PartitionRecords.  */
-	memset (pmbr.PartitionRecord, 0, sizeof pmbr.PartitionRecord);
+	/* Zero out the legacy partitions.  */
+	memset (pmbr->PartitionRecord, 0, sizeof pmbr->PartitionRecord);
 
-	pmbr.Signature = PED_CPU_TO_LE16(MSDOS_MBR_SIGNATURE);
-	pmbr.PartitionRecord[0].OSType      = EFI_PMBR_OSTYPE_EFI;
-	pmbr.PartitionRecord[0].StartSector = 1;
-	pmbr.PartitionRecord[0].EndHead     = 0xFE;
-	pmbr.PartitionRecord[0].EndSector   = 0xFF;
-	pmbr.PartitionRecord[0].EndTrack    = 0xFF;
-	pmbr.PartitionRecord[0].StartingLBA = PED_CPU_TO_LE32(1);
+	pmbr->Signature = PED_CPU_TO_LE16(MSDOS_MBR_SIGNATURE);
+	pmbr->PartitionRecord[0].OSType      = EFI_PMBR_OSTYPE_EFI;
+	pmbr->PartitionRecord[0].StartSector = 1;
+	pmbr->PartitionRecord[0].EndHead     = 0xFE;
+	pmbr->PartitionRecord[0].EndSector   = 0xFF;
+	pmbr->PartitionRecord[0].EndTrack    = 0xFF;
+	pmbr->PartitionRecord[0].StartingLBA = PED_CPU_TO_LE32(1);
 	if ((dev->length - 1ULL) > 0xFFFFFFFFULL)
-		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(0xFFFFFFFF);
+		pmbr->PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(0xFFFFFFFF);
 	else
-		pmbr.PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(dev->length - 1UL);
+		pmbr->PartitionRecord[0].SizeInLBA = PED_CPU_TO_LE32(dev->length - 1UL);
 
-	return ped_device_write (dev, &pmbr, GPT_PMBR_LBA, GPT_PMBR_SECTORS);
+        int write_ok = ped_device_write (dev, pmbr, GPT_PMBR_LBA,
+                                         GPT_PMBR_SECTORS);
+        free (s0);
+	return write_ok;
 }
 
 static void
@@ -1132,6 +1136,7 @@ gpt_write(const PedDisk * disk)
 	free (ptes);
 	return ped_device_sync (disk->dev);
 
+	free (pth_raw);
 error_free_ptes:
 	free (ptes);
 error:
@@ -1283,7 +1288,7 @@ gpt_partition_set_system (PedPartition* part, const PedFileSystemType* fs_type)
 	if (fs_type) {
 		if (strncmp (fs_type->name, "fat", 3) == 0
 		    || strcmp (fs_type->name, "ntfs") == 0) {
-			gpt_part_data->type = PARTITION_MSFT_RESERVED_GUID;
+			gpt_part_data->type = PARTITION_BASIC_DATA_GUID;
 			return 1;
 		}
 		if (strncmp (fs_type->name, "hfs", 3) == 0) {
@@ -1578,6 +1583,12 @@ gpt_partition_align (PedPartition* part, const PedConstraint* constraint)
 	return 0;
 }
 
+static bool
+gpt_partition_check (const PedPartition* part)
+{
+	return true;
+}
+
 static PedDiskOps gpt_disk_ops = {
 	probe:		gpt_probe,
 #ifndef DISCOVER_ONLY
@@ -1605,6 +1616,7 @@ static PedDiskOps gpt_disk_ops = {
 	partition_get_name:		gpt_partition_get_name,
 	partition_align:		gpt_partition_align,
 	partition_enumerate:		gpt_partition_enumerate,
+	partition_check:		gpt_partition_check,
 	alloc_metadata:			gpt_alloc_metadata,
 	get_max_primary_partition_count: gpt_get_max_primary_partition_count,
 	get_max_supported_partition_count: gpt_get_max_supported_partition_count
