@@ -1,7 +1,6 @@
 /*
     parted - a frontend to libparted
-    Copyright (C) 1999, 2000, 2001, 2002, 2006, 2007, 2008, 2009
-    Free Software Foundation, Inc.
+    Copyright (C) 1999-2002, 2006-2009 Free Software Foundation, Inc.
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,6 +15,7 @@
     You should have received a copy of the GNU General Public License
     along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
+#include <config.h>
 
 #include <config.h>
 
@@ -28,6 +28,7 @@
 #include <string.h>
 #include <unistd.h>
 #include <setjmp.h>
+#include <assert.h>
 
 #include "command.h"
 #include "strlist.h"
@@ -841,8 +842,12 @@ command_line_prompt_words (const char* prompt, const char* def,
                 real_prompt = _construct_prompt (prompt, _def, possibilities);
                 line = _readline (real_prompt, possibilities);
                 free (real_prompt);
-                if (!line)
+                if (!line) {
+                        /* readline returns NULL to indicate EOF.
+                           Treat that like an interrupt.  */
+                        got_ctrl_c = 1;
                         break;
+                }
 
                 if (!strlen (line)) {
                         if (_def)
@@ -979,15 +984,12 @@ command_line_get_state (const char* prompt, int* value)
 int
 command_line_get_device (const char* prompt, PedDevice** value)
 {
-        char*         def_dev_name = *value ? (*value)->path : NULL;
-        char*         dev_name;
-        PedDevice*    dev;
-
-        dev_name = command_line_get_word (prompt, def_dev_name, NULL, 1);
+        char *def_dev_name = *value ? (*value)->path : NULL;
+        char *dev_name = command_line_get_word (prompt, def_dev_name, NULL, 1);
         if (!dev_name)
                 return 0;
 
-        dev = ped_device_get (dev_name);
+        PedDevice *dev = ped_device_get (dev_name);
         free (dev_name);
         if (!dev)
                 return 0;
@@ -1004,6 +1006,7 @@ command_line_get_disk (const char* prompt, PedDisk** value)
         if (!command_line_get_device (prompt, &dev))
                 return 0;
 
+        assert (*value);
         if (dev != (*value)->dev) {
                 PedDisk*    new_disk = ped_disk_new (dev);
                 if (!new_disk)
@@ -1388,6 +1391,19 @@ init_disk_type_str ()
         return 1;
 }
 
+int
+init_readline (void)
+{
+#ifdef HAVE_LIBREADLINE
+  if (!opt_script_mode) {
+    rl_initialize ();
+    rl_attempted_completion_function = (CPPFunction*) complete_function;
+    readline_state.in_readline = 0;
+  }
+#endif
+  return 0;
+}
+
 int
 init_ui ()
 {
@@ -1398,12 +1414,6 @@ init_ui ()
                 return 0;
         ped_exception_set_handler (exception_handler);
 
-#ifdef HAVE_LIBREADLINE
-        rl_initialize ();
-        rl_attempted_completion_function = (CPPFunction*) complete_function;
-        readline_state.in_readline = 0;
-#endif
-
 #ifdef SA_SIGINFO
         sigset_t curr;
         sigfillset (&curr);
